# coding: utf-8

"""
    VRt.Packer [PC]

    Veeroute Packer API.  # Description  The service is designed to calculate the placement of goods inside given packages.  ## Types of packages  ### Pallet  ![pallet](../images/packer_pallet.svg)  *Mono-pallet*  If the pallet is fully equipped with the same product, then such a pallet is called **mono-pallet**, such a pallet has the following loading rules:  1. The maximum allowable height of the pallet with the goods laid on it is taken into account. 1. The maximum allowable weight of the loaded pallet is taken into account. 1. Goods packages can only be stacked parallel to the walls of the pallet. 1. It is allowed to rotate packages around the vertical Z-axis and only at a right angle. 1. In order to equalize the height between the positions (to place another pallet on top), empty spaces can be added to the stacking by layers.  *Zone tray*  After the formation of mono-pallets, the remaining part of the goods of the same article is placed on the next pallet, which is divided into zones by means of a cardboard partition in such a way that only one item of goods is placed in each zone, while the zones have the following restrictions:  1. The zones are rectangular. 1. The ratio between the sides of the zones can be any. 1. One of the sides of any zone is always in contact with the edge of the pallet. 1. Permissible number of zones on a pallet: from 2 to 6. 1. Each zone is filled according to the rules for filling a mono-pallet. 1. The stacking height of all goods does not exceed 50 mm, so that another pallet can be placed on top of such a pallet. 1. It is allowed to form pallets with different stacking heights for different positions (height difference is more than 50 mm), if these are the top pallets in the stack.  ### Mixbox  ![mixbox](../images/packer_mixbox.svg)  If the above steps fail to place the item in this way, so that the pallet is of the required height or weight, mixboxes are used:  1. Mixbox can only be placed in one layer. 1. Different goods can be placed in one mixbox, in this case it is divided by partitions similarly to a pallet (the only difference is that the height of the packed goods is not aligned). 1. Mixboxes can be mixed with items on the same pallet. 1. One product can be packed in different mixboxes.  ### Slot  ![slot](../images/packer_slot.svg)  The loaded pallets are stacked on top of each other, the slot is where the stack of pallets is placed. The slot has no physical body - so the dimensions of the `body` of the slot are zero.  ## Coordinate system  ![coordinates](../images/packer_coordinates.svg)  To indicate the location of packages and goods, a local coordinate system is used, in which the coordinates of a particular object are indicated relative to the near left lower corner of the parent object, while:  * Horizontal plane - axes **X**, **Y** * Vertical - **Z** axis  The ratio of the axes of goods and packages:  * Width - **X** * Length - **Y** * Height - **Z**  ## Entity diagram  ![erd](../uml/packer.svg)

    The version of the OpenAPI document: 6.17.2224
    Contact: servicedesk@veeroute.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, ClassVar, Dict, List
from pydantic import BaseModel
from vrt_lss_packer.models.coordinates import Coordinates
from vrt_lss_packer.models.dimensions import Dimensions
from vrt_lss_packer.models.product_group_layout import ProductGroupLayout

class PackageLayout(BaseModel):
    """
    The layout of the product or other packaging in the package.   # noqa: E501
    """
    package_key: Annotated[str, Field(min_length=1, strict=True, max_length=1024)] = Field(description="Package key.")
    coordinates: Coordinates
    dimensions: Dimensions
    product_group_layouts: Annotated[List[ProductGroupLayout], Field(min_length=0, max_length=70001)] = Field(description="List of product group layouts.")
    package_layouts: Annotated[List[PackageLayout], Field(min_length=0, max_length=70001)] = Field(description="List of package layouts.")
    __properties = ["package_key", "coordinates", "dimensions", "product_group_layouts", "package_layouts"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PackageLayout:
        """Create an instance of PackageLayout from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of coordinates
        if self.coordinates:
            _dict['coordinates'] = self.coordinates.to_dict()
        # override the default output from pydantic by calling `to_dict()` of dimensions
        if self.dimensions:
            _dict['dimensions'] = self.dimensions.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in product_group_layouts (list)
        _items = []
        if self.product_group_layouts:
            for _item in self.product_group_layouts:
                if _item:
                    _items.append(_item.to_dict())
            _dict['product_group_layouts'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in package_layouts (list)
        _items = []
        if self.package_layouts:
            for _item in self.package_layouts:
                if _item:
                    _items.append(_item.to_dict())
            _dict['package_layouts'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PackageLayout:
        """Create an instance of PackageLayout from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PackageLayout.parse_obj(obj)

        _obj = PackageLayout.parse_obj({
            "package_key": obj.get("package_key"),
            "coordinates": Coordinates.from_dict(obj.get("coordinates")) if obj.get("coordinates") is not None else None,
            "dimensions": Dimensions.from_dict(obj.get("dimensions")) if obj.get("dimensions") is not None else None,
            "product_group_layouts": [ProductGroupLayout.from_dict(_item) for _item in obj.get("product_group_layouts")] if obj.get("product_group_layouts") is not None else None,
            "package_layouts": [PackageLayout.from_dict(_item) for _item in obj.get("package_layouts")] if obj.get("package_layouts") is not None else None
        })
        return _obj

PackageLayout.update_forward_refs()

